#
# zadolbator: simple reminder component for Jabber service
#
# Copyright (C) 2007 Alexander Tsvyashchenko
#
# This program is free software; you can redistribute it and/or modify it under the terms of the GNU
# General Public License as published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
# even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with this program; if not,
# write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

require 'logger'
require 'optparse'
require 'ostruct'

require 'zadolbator/component'

options = OpenStruct.new
options.config = '/etc/jabber/zadolbator.yaml'
options.log = '/var/log/jabber/zadolbator.log'
options.log_level = Logger::WARN

LOG_LEVELS = {
              'DEBUG' => Logger::DEBUG,
              'INFO' => Logger::INFO,
              'WARN' => Logger::WARN,
              'ERROR' => Logger::ERROR,
              'FATAL' => Logger::FATAL
             }

opts = OptionParser.new do |opts|
  opts.banner = "Usage: zadolbator.rb [options]"

  opts.separator ""
  opts.separator "Specific options:"

  opts.on("-c", "--config CONFIGFILE",
          "Config file to use") do |cfg|
    options.config = cfg
  end

  opts.on("-l", "--log LOGFILE",
          "Log file to use") do |log|
    options.log = log
  end

  opts.on("--log-level [LEVEL]", LOG_LEVELS.keys,
          "Select logger level (DEBUG, INFO, WARN, ERROR, FATAL)") do |log_level|
    options.log_level = LOG_LEVELS[log_level]
  end

  opts.separator ""
  opts.separator "Common options:"

  opts.on_tail("-h", "--help", "Show this message") do
    puts opts
    exit
  end

  opts.parse!(ARGV)
  options
end

begin
  t = ZadolbatorComponent.new(options.config, options.log, options.log_level)
  Thread.stop
rescue Interrupt => e
  t.shutdown!
end
