#
# zadolbator: simple reminder component for Jabber service
#
# Copyright (C) 2007 Alexander Tsvyashchenko
#
# This program is free software; you can redistribute it and/or modify it under the terms of the GNU
# General Public License as published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
# even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with this program; if not,
# write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

#
# Common code for forms-based and messages-based interface
#
module AddEventsCmdCommon
  def AddEventsCmdCommon.add_cals(cals, jid)
    added_events = 0
    cals.each { |cal|
      cal.events.each { |event|
        new_cal = Icalendar::Calendar.new
        new_cal.add_event(event)
        e = Storage::Event.new(:ical => new_cal.to_ical, :owner_id => jid)
        e.id = event.uid
        e.save
        added_events += 1
      }
    }
    return added_events
  end
end

#
# Forms-based interface
#
class AddEventsCmd

  @@cmd_name = 'add_events'

  def AddEventsCmd.cmd_name
    @@cmd_name
  end

  def process(iq, log)
    cmd = iq.first_element('command')
    iq.type = :result

    if cmd.attributes['action'] == 'cancel'
      cmd.delete_elements('x')
      cmd.attributes['status'] = 'canceled'
    elsif x = cmd.first_element('x')
      if x.attributes['type'] == 'submit'

        added_events = 0
        cmd.attributes['status'] = 'completed'
        x.attributes['type'] = 'result'

        x.each_element('field') { |field|
          if field.attributes['var'] and field.attributes['var'] == 'Events'

            cals = CommonCmds.parse_cals(field, iq, x)

            if cals != nil
              begin
                added_events += AddEventsCmdCommon.add_cals(cals, Storage::User.find(iq.from.strip.to_s).id)
              rescue => ex
                CommonCmds.error_msg(x, "Error when adding event: #{ex}")
                return false
              end
            end
          end
        }
        x.delete_elements('field')
        x.delete_elements('instructions')
        x.add(instr = Jabber::Dataforms::XDataInstructions.new)
        instr.text = "Success: #{added_events} event(s) added"
        return added_events > 0
      else
        iq.type = :error
        iq.add_element(Jabber::Error.new('bad-request'))
        return false
      end
    else
      cmd.attributes['status'] = 'executing'
      cmd.add(x = Jabber::Dataforms::XData.new(:form))
      x.add(title = Jabber::Dataforms::XDataTitle.new)
      title.text = 'Add Events'
      x.add(instr = Jabber::Dataforms::XDataInstructions.new)
      instr.text = 'Please add new events in iCalendar format'
      x.add(inp = Jabber::Dataforms::XDataField.new('Events', :text_multi))
      inp.required = true
    end
    return false
  end

  def add_cmd(jid, query)
    query.add(Jabber::Discovery::Item::new(jid, 'Add Events', @@cmd_name))
  end
end

#
# Messages-based interface
#

class AddEventsMsgCmd

  @@cmd_name = 'add'
  @@cmd_help = 'send message with events to add in iCalendar format'

  def AddEventsMsgCmd.cmd_name
    @@cmd_name
  end

  def cmd_name
    @@cmd_name
  end

  def AddEventsMsgCmd.cmd_help
    @@cmd_help
  end

  def cmd_help
    @@cmd_help
  end

  def process(msg, log)
    text = StringIO.open(msg.body)
    text.readline # Skip first line - it should be command name
    begin
      cals = Icalendar::parse(text)
    rescue => ex
      msg.body = "Exception while parsing iCalendar input: #{ex}"
      return false
    end

    added_events = 0

    if cals != nil
      begin
        added_events += AddEventsCmdCommon.add_cals(cals, Storage::User.find(msg.from.strip.to_s).id)
      rescue => ex
        msg.body = "Error when adding event: #{ex}"
        return false
      end
    end

    msg.body = "Success: #{added_events} event(s) added"
    return added_events > 0
  end
end