#
# zadolbator: simple reminder component for Jabber service
#
# Copyright (C) 2007 Alexander Tsvyashchenko
#
# This program is free software; you can redistribute it and/or modify it under the terms of the GNU
# General Public License as published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
# even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with this program; if not,
# write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

module CommonCmds
  def CommonCmds.populate_events(x, cond)
    x.add(inp = Jabber::Dataforms::XDataField.new('Events', :list_multi))
    inp.required = true
    opts = Hash.new
    Storage::Event.find(:all, :conditions => cond).each { |e|
      cals = Icalendar::parse(e.ical)
      # We know only one calendar should be present
      cals.first.events.each { |event|
        if event.dtstart.hour() == 0 and event.dtstart.min() == 0 and event.dtstart.sec() == 0
          date = event.dtstart.strftime('%F')
          opts[e.id] = "#{date} - #{event.summary}"
        else
          opts[e.id] = "#{event.dtstart} - #{event.summary}"
        end
      }
    }
   inp.options = opts
   inp
  end

  def CommonCmds.error_msg(x, text)
    x.attributes['type'] = 'result'
    x.delete_elements('title')
    x.delete_elements('field')
    x.delete_elements('instructions')
    x.add(title = Jabber::Dataforms::XDataTitle.new)
    title.text = 'Error'
    x.add(instr = Jabber::Dataforms::XDataInstructions.new)
    instr.text = text
  end

  def CommonCmds.parse_cals(field, iq, x)
    ical_text = ''

    field.each_element('value') { |el|
      ical_text += el.text.chomp + "\n" if el.text
    }

    begin
      cals = Icalendar::parse(ical_text)
    rescue => ex
      CommonCmds.error_msg(x, "Icalendar parsing failure: #{ex}")
#      iq.type = :error
#      iq.add_element(Jabber::Error.new('not-acceptable'))
      return nil
    end

    return cals
  end
end
