#
# zadolbator: simple reminder component for Jabber service
#
# Copyright (C) 2007 Alexander Tsvyashchenko
#
# This program is free software; you can redistribute it and/or modify it under the terms of the GNU
# General Public License as published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
# even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with this program; if not,
# write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

class ImportEventsCmd

  @@cmd_name = 'import_events'

  def ImportEventsCmd.cmd_name
    @@cmd_name
  end

  def process(iq, log)
    cmd = iq.first_element('command')
    owner_id = Storage::User.find(iq.from.strip.to_s).id
    iq.type = :result

    if cmd.attributes['action'] == 'cancel'
      cmd.delete_elements('x')
      cmd.attributes['status'] = 'canceled'
    elsif x = cmd.first_element('x')
      if x.attributes['type'] == 'submit'

        added_events = 0
        cmd.attributes['status'] = 'completed'
        x.attributes['type'] = 'result'

        # First we need to get common fields
        common_fields = ''
        x.each_element('field') { |field|
          if field.attributes['var'] and field.attributes['var'] == 'Common Fields'
            field.each_element('value') { |el|
              common_fields += el.text.chomp + "\n" if el.text
            }
          end
        }

        x.each_element('field') { |field|
          if field.attributes['var'] and field.attributes['var'] == 'Events'

            field.each_element('value') { |el|
              if el.text
                log.debug("Parsing #{el.text}")
                begin
                  if el.text =~ /([^\s]+)\s+(.+)/
                    log.debug("Parsed: #{$1}; #{$2}")
                    time = DateTime.parse($1)
                    log.debug("DateTime: #{time}")
                    new_cal = Icalendar::Calendar.new
                    event = new_cal.event {
                      dtstart time
                      summary $2
                    }

                    res_ical = ''

                    StringIO.open(new_cal.to_ical).each_line { |line|
                      res_ical += line.chomp + "\n"

                      # This is always added by iCalendar, if not present,
                      # so we can rely on its existance
                      if line =~ /^DTSTAMP\:/
                        res_ical += common_fields
                      end
                    }

                    log.debug(res_ical)

                    begin
                      cals = Icalendar::parse(res_ical)
                    rescue => ex
                      CommonCmds.error_msg(x, "Icalendar parsing failure: #{ex}")
                      return false
                    end

                    begin
                      e = Storage::Event.new(:ical => cals.first.to_ical, :owner_id => Storage::User.find(iq.from.strip.to_s).id)
                      e.id = cals.first.events.first.uid
                      e.save
                    rescue => ex
                      CommonCmds.error_msg(x, "Error when adding event: #{ex}")
                      return false
                    end

                    added_events += 1
                  end
                rescue => ex
                  CommonCmds.error_msg(x, "Error when parsing line: #{el.text}, error description: #{ex}")
                  return false
                end
              end
            }
          end
        }
        x.delete_elements('field')
        x.delete_elements('instructions')
        x.add(instr = Jabber::Dataforms::XDataInstructions.new)
        instr.text = "Success: #{added_events} event(s) added"
        return added_events > 0
      else
        iq.type = :error
        iq.add_element(Jabber::Error.new('bad-request'))
        return false
      end
    else
      cmd.attributes['status'] = 'executing'
      cmd.add(x = Jabber::Dataforms::XData.new(:form))
      x.add(title = Jabber::Dataforms::XDataTitle.new)
      title.text = 'Import Events'
      x.add(instr = Jabber::Dataforms::XDataInstructions.new)
      instr.text = "Please add new events in the following format:" +
                   "\n<date> <event>\n\n" +
                   "You can also specify common fields you want to add to each iCalendar entity created.\n" +
                   "Minimally, you should specify alarms."
      x.add(inp = Jabber::Dataforms::XDataField.new('Events', :text_multi))
      inp.required = true
      x.add(inp = Jabber::Dataforms::XDataField.new('Common Fields', :text_multi))
      inp.required = true
    end
    return false
  end

  def add_cmd(jid, query)
    query.add(Jabber::Discovery::Item::new(jid, 'Import Events', @@cmd_name))
  end
end
